﻿"uses strict";
// #region Declarations 

var express = require('express');
var ejs = require('ejs');
var basicauth = require('express-basic-auth');
var app = express();
    app.engine('html', ejs.renderFile);
    app.set('views','./templates');
    app.use(express.static('./'));
    app.use('/public', express.static('./public/'));
    app.use('/jquery', express.static('./node_modules/jquery/dist'));
    app.use('/socket.io', express.static('./node_modules/socket.io-client/dist/')); 
    app.use(basicauth({users: { 'bud': 'spencer' },challenge: true }));

var httpserver = require('http').createServer(app);
var socket = require('socket.io').listen(httpserver);

var rpio = require("rpio");
    rpio.init({ gpiomem: false, mapping: 'gpio' });
    rpio.spiBegin();
    rpio.spiSetDataMode(0);

var spi_io = require("./spi_io.js");
var sensor = require("./sensor.js");
var logger = require("./logger.js");

var mcp23s17 = new spi_io.MCP23S17(rpio, 1, 256);
var mcp3208 = new spi_io.MCP3208(rpio, 0, 256);

var inputIo = new sensor.INPUT("./sensor.json", mcp23s17);
var sensorAn = new sensor.ANALOG("./sensor.json", mcp3208);
var outputIo = new sensor.OUTPUT("./sensor.json", mcp23s17);

var datalogger = new logger.DATALOGGER();
var buffer = new logger.BUFFER(72000);
var system = new logger.SYSTEM();

// #endregion


// #region Routes



//Route defaut
app.get('/', function (req, res) {
    res.render("graphique.html");
});

//Route datalogger web-service GET Statut (json)
app.get('/statut', function (req, res) {
    delete sensorAn['Event'];
    delete outputIo['Event'];
    delete outputIo['Event'];
    delete buffer['Event'];
    delete system['Event'];
    res.set('Content-Type', 'application/json');
    res.json({ sensorAn: sensorAn, inputIo: inputIo, outputIo: outputIo, buffer: buffer, system: system });
    res.end();
});

//Route download data buffer
app.get('/logfile', function (req, res) {
    res.set('Content-Type', 'text/csv');
    res.set('Content-Disposition', 'attachment; filename=' + buffer.CsvFileame());
    res.set('Pragma: no-cache');
    res.set('Expires: 0');
    res.status(200).send(buffer.Csv());
    res.end();
});

// #endregion


// #region socket.on

//Evenement connection socket.io
socket.sockets.on("connection", function (client) {
    clientIP = client.request.connection.remoteAddress.replace('::ffff:', '');
    client.emit("Sensors.Config", { sensorAn: sensorAn, inputIo: inputIo, outputIo: outputIo });   
    client.emit("Message", "Welcome to Fabulous Bud NodeJS PI D4T4LOGGER V0.0001 :-)");
    client.emit("Datalogger.Rate", datalogger.RunStatut() ?  datalogger.Rate : "On Pause" );
 
    //Evenement socket.io client vidage du data buffer
    client.on('Buffer.Clear', function () {
        buffer.Clear();
        socket.sockets.emit("Message", "Buffer cleared by " + 
            client.request.connection.remoteAddress.replace('::ffff:', '')
        );
    });

    //Evenement socket.io client up dataloger datarate
    client.on('Datalogger.RateUp', function () {
        datalogger.RateUp();
        socket.sockets.emit("Message", "Logger Rate UP by " + 
            client.request.connection.remoteAddress.replace('::ffff:', '')
        );
    });

    //Evenement socket.io client down dataloger datarate
    client.on('Datalogger.RateDown', function () {
        datalogger.RateDown();
        socket.sockets.emit("Message", "Logger Rate Down by " + 
            client.request.connection.remoteAddress.replace('::ffff:', '')
        );
    });

    //Evenement socket.io client dataloger pause/run
    client.on('Datalogger.Running', function () {
        datalogger.Running();
        socket.sockets.emit("Message", "Datalogger " +
            (datalogger.RunStatut() ? "Run" : "Pause") +
            " by " +
            client.request.connection.remoteAddress.replace('::ffff:', '')
        );
    });

    //Evenement socket.io client reboot system
    client.on('System.Reboot', function () {
        socket.sockets.emit("Message", "System Rebooted by " +
            client.request.connection.remoteAddress.replace('::ffff:', '')
        );
        system.Reboot();
    });

    //Evenement socket.io client shutdown system
    client.on('System.Shutdown', function () {
        socket.sockets.emit("Message", "System Stopped by " + 
            client.request.connection.remoteAddress.replace('::ffff:', '')
        );
        system.Shutdown();
    });

});

// #endregion


// #region Evenements des objets

//Evenement message de l'objet system
system.Event.on('System.Message', function (message) {
    socket.sockets.emit('Message', message);
});

//Evenement valeurs des entrees de l'objet inputIo
inputIo.Event.on('Input.Values', function () {
    socket.sockets.emit('Input.Values', {Input: inputIo.Values});
});

//Evenement valeurs des sorties de l'objet outputIo
outputIo.Event.on('Output.Values', function () {
    socket.sockets.emit('Output.Values', {Output: outputIo.Values});
});

//Evenement valeurs des entrees analogiques de l'objet sensorAn
sensorAn.Event.on('Analog.Values', function () {
    socket.sockets.emit('Analog.Values', {Analog: sensorAn.Values});
});

//Evenement alarme des entrees analogiques de l'objet sensorAn
sensorAn.Event.on('Analog.Alarme', function (channel) {
    outputIo.Write(channel, sensorAn.Channel[channel].Alarme);
});

//Evenement aupdate de l'objet buffer
buffer.Event.on('Buffer.Update', function (size, use) {
    socket.sockets.emit('Buffer.Update', { size: size, use: use });
});

//Evenement datarate de l'objet datalogger
datalogger.Event.on('Datalogger.Rate', function (rate) {
    socket.sockets.emit('Datalogger.Rate', rate);
});


//Evenement ticks de l'objet datalogger
datalogger.Event.on('Datalogger.Ticks', function () {
    sensorAn.Read();
    inputIo.Read();
    buffer.Add(sensorAn.Values, inputIo.Values, outputIo.Values);
});

// #endregion

//Evenement SIGINT (executé a la fermeture du programme)
process.on('SIGINT', function () {
    outputIo.Write(0);
    rpio.spiEnd();
    process.exit();
});

//Demarrage du server http
httpserver.listen(8080);
