﻿"uses strict";
var events = require('events');
var Sensor = {

    ANALOG: function (sensorfile, adconverter) {

        var ch = require(sensorfile).ANALOG;
        var dmax = Math.pow(2, adconverter.Resolution) - 1;
        var chEvent = new events.EventEmitter();
        var vs = [];

        for (let i = 0; i < ch.length; i++) {
            ch[i].Coef = (ch[i].Vmax - ch[i].Vmin) / dmax;
            ch[i].Value = ch[i].Vmin; 
            ch[i].Alarme = false;
        }
      
        this.Read = function () {
            let adcr = adconverter.ReadAll();
            for (let i = 0; i < ch.length; i++) {
                let alarme = ch[i].Alarme;
                ch[i].Value = (ch[i].Coef * adcr[i] + ch[i].Vmin).toFixed(ch[i].Dec); 
                vs[i] = ch[i].Value;
                ch[i].Alarme = ch[i].Value > ch[i].Valm; 
                if (alarme !== ch[i].Alarme) {
                    chEvent.emit("Analog.Alarme", i);
                }
            }  
            chEvent.emit("Analog.Values");
        };

        this.Values = vs;
        this.Channel = ch;
        this.Event = chEvent;
    }

    ,   

    INPUT: function (sensorfile, expander) {

        var pin = require(sensorfile).INPUT; 
        var pinEvent = new events.EventEmitter();
        var vs = [];

        for (let i = 0; i < pin.length; i++) {
            pin[i].Value = false;
            vs[i] = false;
        }
       
        this.Read = function () {
            let expr = expander.ReadInput();
            for (let i = 0; i < pin.length; i++) {
                pin[i].Value = Boolean(expr & 1 << i);
                vs[i] = pin[i].Value;
            }
            pinEvent.emit("Input.Values");
        };
        this.Read();

        this.Values = vs;
        this.Pin = pin;
        this.Event = pinEvent;

    }

    ,

    OUTPUT: function (sensorfile, expander) {

        var pin = require(sensorfile).OUTPUT;
        var pinEvent = new events.EventEmitter();
        var vs = [];
        var vsi = 0;
        expander.WriteOutput(vsi);

        this.Read = function () {
            vsi = expander.ReadOutput();
            for (let i = 0; i < pin.length; i++) {
                pin[i].Value = Boolean(vsi & 1 << i);
                vs[i] = pin[i].Value;
            }
            pinEvent.emit("Output.Values");          
        };

        this.Write = function (iopin, boolval) {
            let v = 1 << iopin;
            if (boolval && !pin[iopin].Value) {
                expander.WriteOutput(vsi += v);
            }
            else {
                if (!boolval && pin[iopin].Value) {
                    expander.WriteOutput(vsi -= v);
                }
            }            
            this.Read();
        };

        this.Read();
        this.Value = vsi;
        this.Values = vs;
        this.Pin = pin;
        this.Event = pinEvent;     
    }

};
module.exports = Sensor;
